@@ -7,49 +7,102 @@
 package azidentity
 
 import (
-	"bytes"
 	"context"
 	"fmt"
 	"net/http"
 	"net/url"
 	"os"
-	"os/exec"
 	"path/filepath"
 	"runtime"
 	"strings"
+	"sync"
 	"testing"
 	"time"
 
 	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
 	"github.com/Azure/azure-sdk-for-go/sdk/azcore/policy"
 	azruntime "github.com/Azure/azure-sdk-for-go/sdk/azcore/runtime"
+	"github.com/Azure/azure-sdk-for-go/sdk/internal/log"
 	"github.com/Azure/azure-sdk-for-go/sdk/internal/mock"
 	"github.com/Azure/azure-sdk-for-go/sdk/internal/recording"
 	"github.com/stretchr/testify/require"
 )
 
-const (
-	expiresOnIntResp          = `{"access_token": "new_token", "refresh_token": "", "expires_in": "", "expires_on": "1560974028", "not_before": "1560970130", "resource": "https://vault.azure.net", "token_type": "Bearer"}`
-	expiresOnNonStringIntResp = `{"access_token": "new_token", "refresh_token": "", "expires_in": "", "expires_on": 1560974028, "not_before": "1560970130", "resource": "https://vault.azure.net", "token_type": "Bearer"}`
+var (
+	// liveScopes is scopes to use for live managed identity tests. Each such test must use a different
+	// scope because MSAL caches managed identity tokens at the level of the process. If two tests use
+	// the same scope, the second will get cached tokens and not send a request.
+	liveScopes = []string{
+		"https://graph.microsoft.com/.default",
+		"https://management.azure.com/.default",
+		"https://storage.azure.com/.default",
+		"https://vault.azure.net/.default",
+	}
+	liveScopesMtx = &sync.Mutex{}
 )
 
-func TestManagedIdentityCredential_AzureArc(t *testing.T) {
-	d := t.TempDir()
-	before := arcKeyDirectory
-	arcKeyDirectory = func() (string, error) { return d, nil }
-	defer func() { arcKeyDirectory = before }()
-	file, err := os.Create(filepath.Join(d, "arc.key"))
-	if err != nil {
-		t.Fatal(err)
+// recordMITest starts a managed identity test recording and returns ClientOptions and a scope for the test
+func recordMITest(t *testing.T) (azcore.ClientOptions, string) {
+	opts, stop := initRecording(t)
+	t.Cleanup(stop)
+	recordedScope := strings.ReplaceAll(t.Name(), "/", "_")
+	if recording.GetRecordMode() == recording.PlaybackMode {
+		return opts, recordedScope
 	}
-	defer file.Close()
-	expectedKey := "expected-key"
-	n, err := file.WriteString(expectedKey)
-	if n != len(expectedKey) || err != nil {
-		t.Fatalf("failed to write key file: %v", err)
+	liveScopesMtx.Lock()
+	defer liveScopesMtx.Unlock()
+	if len(liveScopes) == 0 {
+		// fail instead of skipping because this would be a bug
+		t.Fatal("all live managed identity test scopes have been used")
 	}
+	scope := liveScopes[0]
+	liveScopes = liveScopes[1:]
+	if recording.GetRecordMode() == recording.RecordingMode {
+		actual := url.QueryEscape(strings.TrimSuffix(scope, defaultSuffix))
+		err := recording.AddURISanitizer(recordedScope, actual, &recording.RecordingOptions{
+			ProxyPort:    os.Getpid()%10000 + 20000, // TODO
+			TestInstance: t,
+			UseHTTPS:     true,
+		})
+		require.NoError(t, err)
+	}
+	return opts, scope
+}
 
+func writeArcKeyFile(t *testing.T, content string) string {
+	d := ""
+	switch o := runtime.GOOS; o {
+	case "linux":
+		d = "/var/opt/azcmagent/tokens"
+	case "windows":
+		pd := os.Getenv("ProgramData")
+		if pd == "" {
+			t.Fatal("environment variable ProgramData has no value")
+		}
+		d = filepath.Join(pd, "AzureConnectedMachineAgent", "Tokens")
+	default:
+		t.Skipf("unsupported OS %q", o)
+	}
+	if _, err := os.Stat(d); err != nil {
+		if err = os.MkdirAll(d, 0755); err != nil {
+			t.Skipf("failed to create Arc key directory: %v", err)
+		}
+		t.Cleanup(func() { _ = os.RemoveAll(d) })
+	}
+	p := filepath.Join(d, "arc.key")
+	err := os.WriteFile(p, []byte(content), 0600)
+	if err != nil {
+		t.Skipf("failed to write Arc key file: %v", err)
+	}
+	t.Cleanup(func() { _ = os.Remove(p) })
+	return p
+}
+
+func TestManagedIdentityCredential_AzureArc(t *testing.T) {
+	expectedKey := "expected-key"
+	fp := writeArcKeyFile(t, expectedKey)
 	expectedPath := "/foo/token"
+	expectedScope := t.Name() + "/.default"
 	validateReq := func(req *http.Request) bool {
 		if req.URL.Path != expectedPath {
 			t.Fatalf("unexpected path: %s", req.URL.Path)
@@ -57,7 +110,7 @@ func TestManagedIdentityCredential_AzureArc(t *testing.T) {
 		if p := req.URL.Query().Get("api-version"); p != azureArcAPIVersion {
 			t.Fatalf("unexpected api-version: %s", p)
 		}
-		if p := req.URL.Query().Get("resource"); p != strings.TrimSuffix(liveTestScope, defaultSuffix) {
+		if p := req.URL.Query().Get("resource"); p != strings.TrimSuffix(expectedScope, defaultSuffix) {
 			t.Fatalf("unexpected resource: %s", p)
 		}
 		if h := req.Header.Get("metadata"); h != "true" {
@@ -71,7 +124,7 @@ func TestManagedIdentityCredential_AzureArc(t *testing.T) {
 
 	srv, close := mock.NewServer()
 	defer close()
-	srv.AppendResponse(mock.WithHeader("WWW-Authenticate", "Basic realm="+file.Name()), mock.WithStatusCode(401))
+	srv.AppendResponse(mock.WithHeader("WWW-Authenticate", "Basic realm="+fp), mock.WithStatusCode(401))
 	srv.AppendResponse(mock.WithPredicate(validateReq), mock.WithBody(accessTokenRespSuccess))
 	srv.AppendResponse()
 
@@ -84,7 +137,7 @@ func TestManagedIdentityCredential_AzureArc(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	testGetTokenSuccess(t, cred)
+	testGetTokenSuccess(t, cred, expectedScope)
 }
 
 func TestManagedIdentityCredential_AzureArcErrors(t *testing.T) {
@@ -116,7 +169,7 @@ func TestManagedIdentityCredential_AzureArcErrors(t *testing.T) {
 			if err != nil {
 				t.Fatal(err)
 			}
-			_, err = cred.GetToken(context.Background(), testTRO)
+			_, err = cred.GetToken(context.Background(), policy.TokenRequestOptions{Scopes: []string{t.Name()}})
 			if err == nil {
 				t.Fatal("expected an error")
 			}
@@ -135,7 +188,7 @@ func TestManagedIdentityCredential_AzureArcErrors(t *testing.T) {
 		if err != nil {
 			t.Fatal(err)
 		}
-		_, err = cred.GetToken(context.Background(), testTRO)
+		_, err = cred.GetToken(context.Background(), policy.TokenRequestOptions{Scopes: []string{t.Name()}})
 		if err == nil {
 			t.Fatal("expected an error")
 		}
@@ -151,19 +204,15 @@ func TestManagedIdentityCredential_AzureArcErrors(t *testing.T) {
 		if err != nil {
 			t.Fatal(err)
 		}
-		_, err = cred.GetToken(context.Background(), testTRO)
+		_, err = cred.GetToken(context.Background(), policy.TokenRequestOptions{Scopes: []string{t.Name()}})
+		// TODO: check for specific error
 		if err == nil {
 			t.Fatal("expected an error")
 		}
 	})
 	t.Run("key too large", func(t *testing.T) {
-		d := t.TempDir()
-		f := filepath.Join(d, "test.key")
-		err := os.WriteFile(f, bytes.Repeat([]byte("."), 4097), 0600)
-		require.NoError(t, err)
-		before := arcKeyDirectory
-		arcKeyDirectory = func() (string, error) { return d, nil }
-		defer func() { arcKeyDirectory = before }()
+		size := 4097
+		f := writeArcKeyFile(t, strings.Repeat(" ", size))
 		srv, close := mock.NewServer(mock.WithTransformAllRequestsToTestServerUrl())
 		defer close()
 		srv.AppendResponse(
@@ -172,14 +221,12 @@ func TestManagedIdentityCredential_AzureArcErrors(t *testing.T) {
 		)
 		cred, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{ClientOptions: azcore.ClientOptions{Transport: srv}})
 		require.NoError(t, err)
-		_, err = cred.GetToken(ctx, testTRO)
-		require.ErrorContains(t, err, "too large")
+		_, err = cred.GetToken(ctx, policy.TokenRequestOptions{Scopes: []string{t.Name()}})
+		require.ErrorContains(t, err, fmt.Sprint(size))
 	})
 	t.Run("unexpected file paths", func(t *testing.T) {
-		d, err := arcKeyDirectory()
-		if err != nil {
-			// test is running on an unsupported OS e.g. darwin
-			t.Skip(err)
+		if n := runtime.GOOS; n != "linux" && n != "windows" {
+			t.Skipf("unsupported OS %q", n)
 		}
 		srv, close := mock.NewServer(mock.WithTransformAllRequestsToTestServerUrl())
 		defer close()
@@ -191,55 +238,35 @@ func TestManagedIdentityCredential_AzureArcErrors(t *testing.T) {
 		o := ManagedIdentityCredentialOptions{ClientOptions: azcore.ClientOptions{Transport: srv}}
 		cred, err := NewManagedIdentityCredential(&o)
 		require.NoError(t, err)
-		_, err = cred.GetToken(ctx, testTRO)
-		require.ErrorContains(t, err, "unexpected file path")
+		_, err = cred.GetToken(ctx, policy.TokenRequestOptions{Scopes: []string{t.Name()}})
+		require.ErrorContains(t, err, "invalid file path")
 
 		srv.AppendResponse(
 			// unexpected extension
-			mock.WithHeader("WWW-Authenticate", "Basic realm="+filepath.Join(d, "foo")),
+			mock.WithHeader("WWW-Authenticate", "Basic realm="+filepath.Join(t.TempDir(), "foo")),
 			mock.WithStatusCode(http.StatusUnauthorized),
 		)
 		cred, err = NewManagedIdentityCredential(&o)
 		require.NoError(t, err)
-		_, err = cred.GetToken(ctx, testTRO)
-		require.ErrorContains(t, err, "unexpected file path")
+		_, err = cred.GetToken(ctx, policy.TokenRequestOptions{Scopes: []string{t.Name()}})
+		require.ErrorContains(t, err, "invalid file")
 	})
-	if runtime.GOOS == "windows" {
-		t.Run("ProgramData not set", func(t *testing.T) {
-			t.Setenv("ProgramData", "")
-			srv, close := mock.NewServer(mock.WithTransformAllRequestsToTestServerUrl())
-			defer close()
-			srv.AppendResponse(
-				mock.WithHeader("WWW-Authenticate", "Basic realm=foo"),
-				mock.WithStatusCode(http.StatusUnauthorized),
-			)
-			cred, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{ClientOptions: azcore.ClientOptions{Transport: srv}})
-			require.NoError(t, err)
-			_, err = cred.GetToken(ctx, testTRO)
-			require.ErrorContains(t, err, "ProgramData")
-		})
-	}
 }
 
 func TestManagedIdentityCredential_AzureContainerInstanceLive(t *testing.T) {
 	// This test triggers the managed identity test app deployed to an Azure Container Instance.
 	// See the bicep file and test resources scripts for details.
-	// It triggers the app with az because the test subscription prohibits opening ports to the internet.
-	name := os.Getenv("AZIDENTITY_ACI_NAME")
-	rg := os.Getenv("AZIDENTITY_RESOURCE_GROUP")
-	if name == "" || rg == "" {
-		t.Skip("set AZIDENTITY_ACI_NAME and AZIDENTITY_RESOURCE_GROUP to run this test")
+	ip := os.Getenv("AZIDENTITY_ACI_IP")
+	if ip == "" {
+		t.Skip("set AZIDENTITY_ACI_IP to run this test")
+	}
+	res, err := http.Get("http://" + ip)
+	require.NoError(t, err)
+	if res.StatusCode != http.StatusOK {
+		b, err := azruntime.Payload(res)
+		require.NoError(t, err)
+		t.Fatal("test application returned an error: " + string(b))
 	}
-	ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
-	defer cancel()
-	command := fmt.Sprintf("az container exec -g %s -n %s --exec-command 'wget -qO- localhost'", rg, name)
-	// using "script" as a workaround for "az container exec" requiring a tty
-	// https://github.com/Azure/azure-cli/issues/17530
-	cmd := exec.CommandContext(ctx, "script", "-q", "-O", "/dev/null", "-c", command)
-	b, err := cmd.CombinedOutput()
-	s := string(b)
-	require.NoError(t, err, s)
-	require.Equal(t, "test passed", s)
 }
 
 func TestManagedIdentityCredential_AzureFunctionsLive(t *testing.T) {
@@ -278,13 +305,12 @@ func TestManagedIdentityCredential_AzureMLLive(t *testing.T) {
 			t.Skip("no value for " + strings.Join(missing, ", "))
 		}
 	}
-	opts, stop := initRecording(t)
-	defer stop()
+	opts, scope := recordMITest(t)
 	cred, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{ClientOptions: opts})
 	if err != nil {
 		t.Fatal(err)
 	}
-	testGetTokenSuccess(t, cred)
+	testGetTokenSuccess(t, cred, scope)
 }
 
 func TestManagedIdentityCredential_CloudShell(t *testing.T) {
@@ -293,7 +319,7 @@ func TestManagedIdentityCredential_CloudShell(t *testing.T) {
 		if err != nil {
 			t.Fatal(err)
 		}
-		if v := req.FormValue("resource"); v != strings.TrimSuffix(liveTestScope, defaultSuffix) {
+		if v := req.FormValue("resource"); v != strings.TrimSuffix(t.Name(), defaultSuffix) {
 			t.Fatalf("unexpected resource: %s", v)
 		}
 		if h := req.Header.Get("metadata"); h != "true" {
@@ -307,50 +333,57 @@ func TestManagedIdentityCredential_CloudShell(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	testGetTokenSuccess(t, msiCred)
+	testGetTokenSuccess(t, msiCred, t.Name())
 }
 
 func TestManagedIdentityCredential_AppService(t *testing.T) {
 	expectedID := "expected-ID"
 	expectedHeader := "header"
-	for _, id := range []ManagedIDKind{ClientID(expectedID), ResourceID(expectedID), nil} {
-		validateReq := func(req *http.Request) bool {
-			if h := req.Header.Get("X-IDENTITY-HEADER"); h != expectedHeader {
-				t.Fatalf("unexpected X-IDENTITY-HEADER: %s", h)
-			}
-			q := req.URL.Query()
-			if v := q.Get("api-version"); v != "2019-08-01" {
-				t.Fatalf(`unexpected api-version "%s"`, v)
-			}
-			if v := q.Get("resource"); v != strings.TrimSuffix(liveTestScope, "/.default") {
-				t.Fatalf(`unexpected resource "%s"`, v)
-			}
-			if id == nil {
-				if q.Get(qpClientID) != "" || q.Get(miResID) != "" {
-					t.Fatal("request shouldn't include a user-assigned ID")
+	for _, id := range []ManagedIDKind{ClientID(expectedID), ObjectID(expectedID), ResourceID(expectedID), nil} {
+		scope := fmt.Sprintf("%s/%T/.default", t.Name(), id)
+		validateReq := func(t *testing.T) func(req *http.Request) bool {
+			return func(req *http.Request) bool {
+				if h := req.Header.Get("X-IDENTITY-HEADER"); h != expectedHeader {
+					t.Fatalf("unexpected X-IDENTITY-HEADER: %s", h)
 				}
-			} else {
-				if q.Get(qpClientID) != "" && q.Get(miResID) != "" {
+				q := req.URL.Query()
+				if v := q.Get("api-version"); v != "2019-08-01" {
+					t.Fatalf(`unexpected api-version "%s"`, v)
+				}
+				if v := q.Get("resource"); v != strings.TrimSuffix(scope, "/.default") {
+					t.Fatalf(`unexpected resource "%s"`, v)
+				}
+				clientID := q.Get(qpClientID)
+				resID := q.Get(miResID)
+				objectID := q.Get("object_id")
+				if (clientID != "" && resID != "") || (clientID != "" && objectID != "") || (resID != "" && objectID != "") {
 					t.Fatal("request includes two IDs")
 				}
-				var v string
-				if _, ok := id.(ClientID); ok {
-					v = q.Get(qpClientID)
-				} else if _, ok := id.(ResourceID); ok {
-					v = q.Get(miResID)
-				}
-				if v != id.String() {
-					t.Fatalf(`unexpected id "%s"`, v)
+				if id == nil {
+					if clientID != "" || resID != "" || objectID != "" {
+						t.Fatal("request shouldn't include a user-assigned ID")
+					}
+				} else {
+					actual := clientID
+					switch id.(type) {
+					case ObjectID:
+						actual = objectID
+					case ResourceID:
+						actual = resID
+					}
+					if actual != id.String() {
+						t.Errorf("expected %s, got %q", id.String(), actual)
+					}
 				}
+				return true
 			}
-			return true
 		}
 
 		t.Run(fmt.Sprintf("%T", id), func(t *testing.T) {
 			srv, close := mock.NewServer()
 			defer close()
 			srv.AppendResponse(
-				mock.WithPredicate(validateReq),
+				mock.WithPredicate(validateReq(t)),
 				mock.WithBody([]byte(fmt.Sprintf(
 					`{"access_token": "%s", "expires_on": "%d", "resource": "https://vault.azure.net", "token_type": "Bearer", "client_id": "some-guid"}`,
 					tokenValue,
@@ -365,7 +398,7 @@ func TestManagedIdentityCredential_AppService(t *testing.T) {
 			if err != nil {
 				t.Fatal(err)
 			}
-			testGetTokenSuccess(t, cred)
+			testGetTokenSuccess(t, cred, scope)
 		})
 	}
 }
@@ -381,7 +414,7 @@ func TestManagedIdentityCredential_AppServiceError(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
-	_, err = msiCred.GetToken(context.Background(), testTRO)
+	_, err = msiCred.GetToken(context.Background(), policy.TokenRequestOptions{Scopes: []string{t.Name()}})
 	if err == nil {
 		t.Fatalf("Expected an error but did not receive one")
 	}
@@ -402,7 +435,7 @@ func TestManagedIdentityCredential_GetTokenIMDS400(t *testing.T) {
 	}
 	// cred should return credentialUnavailableError when IMDS responds 400 to a token request
 	for i := 0; i < 3; i++ {
-		_, err = cred.GetToken(context.Background(), testTRO)
+		_, err = cred.GetToken(context.Background(), policy.TokenRequestOptions{Scopes: []string{t.Name()}})
 		if _, ok := err.(credentialUnavailable); !ok {
 			t.Fatalf("expected credentialUnavailable, received %T", err)
 		}
@@ -437,45 +470,12 @@ func TestManagedIdentityCredential_GetTokenUnexpectedJSON(t *testing.T) {
 	if err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
-	_, err = msiCred.GetToken(context.Background(), testTRO)
+	_, err = msiCred.GetToken(context.Background(), policy.TokenRequestOptions{Scopes: []string{t.Name()}})
 	if err == nil {
 		t.Fatalf("Expected a JSON marshal error but received nil")
 	}
 }
 
-func TestManagedIdentityCredential_CreateIMDSAuthRequest(t *testing.T) {
-	cred, err := NewManagedIdentityCredential(nil)
-	if err != nil {
-		t.Fatalf("unexpected error: %v", err)
-	}
-	req, err := cred.mic.createIMDSAuthRequest(context.Background(), ClientID(fakeClientID), []string{liveTestScope})
-	if err != nil {
-		t.Fatal(err)
-	}
-	if req.Raw().Header.Get(headerMetadata) != "true" {
-		t.Fatalf("Unexpected value for Content-Type header")
-	}
-	reqQueryParams, err := url.ParseQuery(req.Raw().URL.RawQuery)
-	if err != nil {
-		t.Fatalf("Unable to parse IMDS query params: %v", err)
-	}
-	if reqQueryParams["api-version"][0] != imdsAPIVersion {
-		t.Fatalf("Unexpected IMDS API version")
-	}
-	if reqQueryParams["resource"][0] != liveTestScope {
-		t.Fatalf("Unexpected resource in resource query param")
-	}
-	if reqQueryParams["client_id"][0] != fakeClientID {
-		t.Fatalf("Unexpected client ID. Expected: %s, Received: %s", fakeClientID, reqQueryParams["client_id"][0])
-	}
-	if u := req.Raw().URL.String(); !strings.HasPrefix(u, imdsEndpoint) {
-		t.Fatalf("Unexpected default authority host %s", u)
-	}
-	if req.Raw().URL.Scheme != "http" {
-		t.Fatalf("Wrong request scheme")
-	}
-}
-
 func TestManagedIdentityCredential_GetTokenScopes(t *testing.T) {
 	srv, close := mock.NewServer()
 	defer close()
@@ -514,46 +514,25 @@ func TestManagedIdentityCredential_ScopesImmutable(t *testing.T) {
 	}
 }
 
-func TestManagedIdentityCredential_ResourceID_IMDS(t *testing.T) {
-	resID := "sample/resource/id"
-	cred, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{ID: ResourceID(resID)})
-	if err != nil {
-		t.Fatalf("unexpected error: %v", err)
-	}
-	req, err := cred.mic.createAuthRequest(context.Background(), cred.mic.id, []string{liveTestScope})
-	if err != nil {
-		t.Fatal(err)
-	}
-	reqQueryParams, err := url.ParseQuery(req.Raw().URL.RawQuery)
-	if err != nil {
-		t.Fatalf("Unable to parse App Service request query params: %v", err)
-	}
-	if reqQueryParams["api-version"][0] != "2018-02-01" {
-		t.Fatalf("Unexpected App Service API version")
-	}
-	if reqQueryParams["resource"][0] != liveTestScope {
-		t.Fatalf("Unexpected resource in resource query param")
-	}
-	if reqQueryParams[msiResID][0] != resID {
-		t.Fatalf("Unexpected resource ID in resource query param")
-	}
-}
-
-func TestManagedIdentityCredential_CreateAccessTokenExpiresOnInt(t *testing.T) {
+func TestManagedIdentityCredential_ExpiresOnInt(t *testing.T) {
 	srv, close := mock.NewServer()
 	defer close()
-	srv.AppendResponse(mock.WithBody([]byte(expiresOnNonStringIntResp)))
+	expires := time.Now().Add(time.Hour).Unix()
+	scope := t.Name()
+	srv.AppendResponse(
+		mock.WithBody([]byte(fmt.Sprintf(
+			`{"access_token":%q,"expires_on":%d,"resource":%q,"token_type":"Bearer"}`, tokenValue, expires, scope,
+		))),
+	)
 	setEnvironmentVariables(t, map[string]string{msiEndpoint: srv.URL()})
 	options := ManagedIdentityCredentialOptions{}
 	options.Transport = srv
 	msiCred, err := NewManagedIdentityCredential(&options)
-	if err != nil {
-		t.Fatalf("unexpected error: %v", err)
-	}
-	_, err = msiCred.GetToken(context.Background(), testTRO)
-	if err != nil {
-		t.Fatal(err)
-	}
+	require.NoError(t, err)
+	tk, err := msiCred.GetToken(ctx, policy.TokenRequestOptions{Scopes: []string{scope}})
+	require.NoError(t, err)
+	require.Equal(t, tokenValue, tk.Token)
+	require.Equal(t, expires, tk.ExpiresOn.Unix())
 }
 
 // adding an incorrect string value in expires_on
@@ -568,7 +547,7 @@ func TestManagedIdentityCredential_CreateAccessTokenExpiresOnFail(t *testing.T)
 	if err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
-	_, err = msiCred.GetToken(context.Background(), testTRO)
+	_, err = msiCred.GetToken(context.Background(), policy.TokenRequestOptions{Scopes: []string{t.Name()}})
 	if err == nil {
 		t.Fatalf("expected to receive an error but received none")
 	}
@@ -583,47 +562,43 @@ func TestManagedIdentityCredential_IMDSLive(t *testing.T) {
 		if recording.GetRecordMode() != recording.PlaybackMode && liveManagedIdentity.clientID == "" {
 			t.Skip("set IDENTITY_VM_USER_ASSIGNED_MI_CLIENT_ID to run this test")
 		}
-		opts, stop := initRecording(t)
-		defer stop()
+		opts, scope := recordMITest(t)
 		cred, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{
 			ClientOptions: opts, ID: ClientID(liveManagedIdentity.clientID)},
 		)
 		require.NoError(t, err)
-		testGetTokenSuccess(t, cred)
+		testGetTokenSuccess(t, cred, scope)
 	})
 
 	t.Run("object ID", func(t *testing.T) {
 		if recording.GetRecordMode() != recording.PlaybackMode && liveManagedIdentity.objectID == "" {
 			t.Skip("set IDENTITY_VM_USER_ASSIGNED_MI_OBJECT_ID to run this test")
 		}
-		opts, stop := initRecording(t)
-		defer stop()
+		opts, scope := recordMITest(t)
 		cred, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{
 			ClientOptions: opts, ID: ObjectID(liveManagedIdentity.objectID)},
 		)
 		require.NoError(t, err)
-		testGetTokenSuccess(t, cred)
+		testGetTokenSuccess(t, cred, scope)
 	})
 
 	t.Run("resource ID", func(t *testing.T) {
 		if recording.GetRecordMode() != recording.PlaybackMode && liveManagedIdentity.resourceID == "" {
 			t.Skip("set IDENTITY_VM_USER_ASSIGNED_MI_RESOURCE_ID to run this test")
 		}
-		opts, stop := initRecording(t)
-		defer stop()
+		opts, scope := recordMITest(t)
 		cred, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{
 			ClientOptions: opts, ID: ResourceID(liveManagedIdentity.resourceID)},
 		)
 		require.NoError(t, err)
-		testGetTokenSuccess(t, cred)
+		testGetTokenSuccess(t, cred, scope)
 	})
 
 	t.Run("system assigned", func(t *testing.T) {
-		opts, stop := initRecording(t)
-		defer stop()
+		opts, scope := recordMITest(t)
 		cred, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{ClientOptions: opts})
 		require.NoError(t, err)
-		testGetTokenSuccess(t, cred)
+		testGetTokenSuccess(t, cred, scope)
 	})
 }
 
@@ -641,7 +616,7 @@ func TestManagedIdentityCredential_IMDSRetries(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	if cred.mic.msiType != msiTypeIMDS {
+	if !cred.mic.imds {
 		t.SkipNow()
 	}
 	for _, code := range []int{404, 410, 429, 500, 501, 502, 503, 504, 505, 506, 507, 508, 510, 511} {
@@ -650,9 +625,9 @@ func TestManagedIdentityCredential_IMDSRetries(t *testing.T) {
 			reqs++
 			return &http.Response{Body: http.NoBody, Request: r, StatusCode: code}
 		}
-		_, err = cred.GetToken(context.Background(), testTRO)
+		_, err = cred.GetToken(context.Background(), policy.TokenRequestOptions{Scopes: []string{fmt.Sprint(code)}})
 		if err == nil {
-			t.Fatal("expected an error")
+			t.Fatalf("expected an error; %d", code)
 		}
 		if reqs != 2 {
 			t.Errorf("expected 1 retry after %d response, got %d", code, reqs-1)
@@ -660,6 +635,37 @@ func TestManagedIdentityCredential_IMDSRetries(t *testing.T) {
 	}
 }
 
+func TestManagedIdentityCredential_Logs(t *testing.T) {
+	logs := []string{}
+	log.SetListener(func(e log.Event, msg string) {
+		if e == EventAuthentication {
+			logs = append(logs, msg)
+		}
+	})
+	defer log.SetListener(nil)
+
+	for _, id := range []ManagedIDKind{ClientID(fakeClientID), ObjectID(fakeObjectID), ResourceID(fakeResourceID), nil} {
+		_, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{ID: id})
+		require.NoError(t, err)
+		require.Len(t, logs, 1)
+		require.Contains(t, logs[0], "IMDS")
+		if id != nil {
+			require.Contains(t, logs[0], id.String())
+			kind := ""
+			switch id.(type) {
+			case ClientID:
+				kind = "client"
+			case ObjectID:
+				kind = "object"
+			case ResourceID:
+				kind = "resource"
+			}
+			require.Contains(t, logs[0], kind+" ID")
+		}
+		logs = nil
+	}
+}
+
 func TestManagedIdentityCredential_UnexpectedIMDSResponse(t *testing.T) {
 	srv, close := mock.NewServer(mock.WithTransformAllRequestsToTestServerUrl())
 	defer close()
@@ -672,7 +678,7 @@ func TestManagedIdentityCredential_UnexpectedIMDSResponse(t *testing.T) {
 	for _, c := range ro.StatusCodes {
 		tests = append(tests, []mock.ResponseOption{mock.WithStatusCode(c)})
 	}
-	for _, res := range tests {
+	for i, res := range tests {
 		srv.AppendResponse(res...)
 
 		c, err := NewManagedIdentityCredential(&ManagedIdentityCredentialOptions{
@@ -683,13 +689,14 @@ func TestManagedIdentityCredential_UnexpectedIMDSResponse(t *testing.T) {
 		})
 		require.NoError(t, err)
 
-		_, err = c.GetToken(ctx, testTRO)
+		_, err = c.GetToken(ctx, policy.TokenRequestOptions{Scopes: []string{t.Name() + fmt.Sprint(i)}})
 		var af *AuthenticationFailedError
 		require.ErrorAs(t, err, &af, "unexpected token response from IMDS should prompt an AuthenticationFailedError")
 	}
 }
 
 func TestManagedIdentityCredential_ServiceFabric(t *testing.T) {
+	scope := t.Name()
 	expectedSecret := "expected-secret"
 	pred := func(req *http.Request) bool {
 		if secret := req.Header.Get("Secret"); secret != expectedSecret {
@@ -698,7 +705,7 @@ func TestManagedIdentityCredential_ServiceFabric(t *testing.T) {
 		if p := req.URL.Query().Get("api-version"); p != serviceFabricAPIVersion {
 			t.Fatalf("unexpected api-version: %s", p)
 		}
-		if p := req.URL.Query().Get("resource"); p != strings.TrimSuffix(liveTestScope, defaultSuffix) {
+		if p := req.URL.Query().Get("resource"); p != scope {
 			t.Fatalf("unexpected resource: %s", p)
 		}
 		return true
@@ -712,7 +719,7 @@ func TestManagedIdentityCredential_ServiceFabric(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	testGetTokenSuccess(t, cred)
+	testGetTokenSuccess(t, cred, scope)
 }
 
 func TestManagedIdentityCredential_UnsupportedID(t *testing.T) {
